package com.dky.generate;

import cn.hutool.json.JSONObject;
import com.dky.calculate.Advantage;
import com.dky.calculate.KitchenCookModel;
import com.dky.calculate.KitchenCookScheme;
import com.dky.modelI.DntdModelI;
import com.dky.utils.CalculateUtils;
import com.dky.utils.GetTargetDeviceList;
import com.dky.utils.GetThisEnergyEmissions;
import com.dky.utils.entity.SysDeviceHeatScene;
import com.dky.utils.enums.DeviceSubType;
import com.dky.utils.enums.EnergyEmissions;
import com.dky.utils.result.MatchedDevice;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class KitchenCookScene implements DntdModelI {

    DecimalFormat decimalFormat = new DecimalFormat("#.00");

    // 首先A=由运行/合同容量折合成kW（x0.9）x85%,
    static final Double COEFFICIENT_1 = 0.9;
    static final Double COEFFICIENT_2 = 0.85;

    @Override
    public JSONObject createReport(JSONObject jsonObject, List<SysDeviceHeatScene> sceneList) {
        List<SysDeviceHeatScene> alternateDeviceList = GetTargetDeviceList.main(DeviceSubType.Kitchen_Cooking.getDesc(), sceneList);

        JSONObject distInfo = new JSONObject();
        JSONObject originalDevInfo = new JSONObject();
        Double runCapacity = null;
        Double lastYearNeed = null;
        Integer peopleNum = null;
        Integer energyCode = null;

        KitchenCookScheme scheme = new KitchenCookScheme();
        KitchenCookModel model = new KitchenCookModel();
        Advantage advantage = new Advantage();

        try{
            distInfo = (JSONObject) jsonObject.get("distInfo");
            originalDevInfo = (JSONObject) jsonObject.get("originalDevInfo");
        } catch (Exception e){
            e.printStackTrace();
        }

        try {
            // 上年最大需量
            lastYearNeed = Double.parseDouble(distInfo.get("lastYearNeed").toString());
        } catch (NullPointerException e1) {
            e1.printStackTrace();
            System.err.println("模型计算必须元素上年最大需量传的为null或者没有传参");
        } catch (NumberFormatException e2) {
            e2.printStackTrace();
            System.err.println("模型计算必须元素上年最大需量为非数字，禁止传参空字符串");
        }

        try {
            // 运行容量
            runCapacity = Double.parseDouble(distInfo.get("runCapacity").toString());
        } catch (NullPointerException e1) {
            e1.printStackTrace();
            System.err.println("模型计算必须元素运行容量传的为null或者没有传参");
        } catch (NumberFormatException e2) {
            e2.printStackTrace();
            System.err.println("模型计算必须元素运行容量为非数字，禁止传参空字符串");
        }

        try {
            // 人数
            peopleNum = Integer.parseInt(jsonObject.get("peopleNum").toString());
        } catch (NullPointerException e1) {
            e1.printStackTrace();
            System.err.println("模型计算必须元素人数为null或者没有传参");
        } catch (NumberFormatException e2) {
            e2.printStackTrace();
            System.err.println("模型计算必须元素人数为非数字，禁止传参空字符串");
        }

        try {
            // 替代前原能源类型编码
            energyCode = (Integer) originalDevInfo.get("energyCode");
        } catch (NullPointerException e1) {
            e1.printStackTrace();
            System.err.println("模型计算必须元素替代前原能源类型编码传的为null或者没有传参");
        } catch (NumberFormatException e2) {
            e2.printStackTrace();
            System.err.println("模型计算必须元素替代前原能源类型编码为非数字，禁止传参空字符串");
        }


        /*
            实际可承载容量A = 运行（或合同容量）x0.9[将运行容量或合同容量折算成容量]x85%
         */
        double A = runCapacity * COEFFICIENT_1 * COEFFICIENT_2;

        /*
            根据人数计算出不同技术类型下所需要不同功率设备数据
         */
        List<MatchedDevice> cookDevices = scheme.getCookDevices(peopleNum, alternateDeviceList);
        /*
          替代后设备总功率C1
         */
        double C1 = 0.0;

        for (MatchedDevice matchedDevice : cookDevices){
            C1 = C1 + matchedDevice.getDeviceHeatScene().getDevPower() * matchedDevice.getCount();
        }

        /*
            改造后最大需量D1
         */
        double D1 = lastYearNeed + C1;

        String remark = "";
        if (A < D1){
            remark = "本方案存在扩容投资需求，扩容投资不计入初次投资费用";
        }


        /*
            封装返回
         */
        JSONObject returnJsonObject = new JSONObject();
        Double startCost                = 0.0;
        Double runCost                  = 0.0;
        Double allCost                  = 0.0;
        Double calculateAnnualCarbon    = 0.0;
        Double laborCost                = 0.0;
        Double electric                 = 0.0;
        List<HashMap<String,Object>> deviceList = new ArrayList<>();
        for (MatchedDevice matchedDevice : cookDevices){
            HashMap<String, Object> map = new HashMap<>();
            map.put("devSubType",matchedDevice.getDeviceHeatScene().getDevSubType());
            map.put("devTechType",matchedDevice.getDeviceHeatScene().getDevTechType());
            map.put("devCount",matchedDevice.getCount());
            map.put("devPrice",matchedDevice.getDeviceHeatScene().getDevPrice());
            deviceList.add(map);

            startCost = startCost + (matchedDevice.getCount() * matchedDevice.getDeviceHeatScene().getDevPrice());

            runCost = runCost + (model.calculateAnnualOperatingCost(matchedDevice.getDeviceHeatScene().getDevPower(), matchedDevice.getCount(), matchedDevice.getDeviceHeatScene().getDevAnnualOperationTime(), matchedDevice.getDeviceHeatScene().getDevSubstituteLaborCost()));

            allCost = allCost + (model.calculateTotalAnnualCost(matchedDevice.getCount(), matchedDevice.getDeviceHeatScene().getDevPrice(), matchedDevice.getDeviceHeatScene().getDevServiceLife(),
                    matchedDevice.getDeviceHeatScene().getDevPower(), matchedDevice.getDeviceHeatScene().getDevAnnualOperationTime(), matchedDevice.getDeviceHeatScene().getDevSubstituteLaborCost()));

            electric = electric + (KitchenCookModel.calculateAnnualElectricityConsumption(matchedDevice.getDeviceHeatScene().getDevPower(), matchedDevice.getCount(), matchedDevice.getDeviceHeatScene().getDevAnnualOperationTime()));

            laborCost = laborCost + (matchedDevice.getCount() * matchedDevice.getDeviceHeatScene().getLaborCost() / matchedDevice.getDeviceHeatScene().getDevServiceLife());

        }

        Double previousYearEnergyUsage = 1.0;
        Double lastYearFee = 0.0;
        Double energyPrice = 0.0;
        try {
            energyPrice = Double.parseDouble(originalDevInfo.get("energyPrice").toString());
        } catch (Exception e) {
            System.out.println("计算因子采用默认值");
            energyPrice = GetThisEnergyEmissions.getThisEnergyPrice(energyCode);
        }

        try {
            // 上年运行费用（元）
            lastYearFee = Double.parseDouble(originalDevInfo.get("lastYearFee").toString());
        } catch (NullPointerException e) {
            previousYearEnergyUsage = Double.parseDouble(originalDevInfo.get("previousYearEnergyUsage").toString());
            lastYearFee = model.calLastYearFee(laborCost, previousYearEnergyUsage, energyPrice);
        } catch (NumberFormatException nfe){
            nfe.printStackTrace();
            System.err.println("上年运行费用（元）为非数字，禁止传参空字符串");
        }

        try {
            // 替代前年能源用量
            previousYearEnergyUsage = Double.parseDouble(originalDevInfo.get("previousYearEnergyUsage").toString());
        } catch (NullPointerException e) {
            try {
                previousYearEnergyUsage = CalculateUtils.divide(lastYearFee, energyPrice);
            } catch (ArithmeticException nfe){
                nfe.printStackTrace();
                System.err.println("数学错误，替代前原使用能源平均单价不可为0");
            }
        } catch (NumberFormatException nfe){
            nfe.printStackTrace();
            System.err.println("替代前年能源用量为非数字，禁止传参空字符串");
        }

        // 替代前上年原设备碳排放量
        double energyLastYearFactor = model.calLastYearFactor(previousYearEnergyUsage, GetThisEnergyEmissions.getThisEnergyEmission(energyCode));
        // 年减碳量
        calculateAnnualCarbon = model.calculateAnnualCarbonReduction(energyLastYearFactor, electric, EnergyEmissions.ELECTRIC.getCo2EmissionFactor());


        //初次投资费用
        returnJsonObject.set("startCost", decimalFormat.format(startCost));
        // 设备总价
        returnJsonObject.set("devCost", decimalFormat.format(startCost));
        //年运行费用
        returnJsonObject.set("yearRunCost", decimalFormat.format(runCost));
        //年总费用
        returnJsonObject.set("yearCost", decimalFormat.format(allCost));
        //年减碳量
        returnJsonObject.set("calculate", decimalFormat.format(calculateAnnualCarbon/1000));
        //替代电量
        returnJsonObject.set("electric", decimalFormat.format(electric));
        //备注
        returnJsonObject.set("remark",remark);
        //封装需配置设备情况
        returnJsonObject.set("deviceList",deviceList);

        //封装方案优势
        returnJsonObject.set("safety", advantage.safety());
        returnJsonObject.set("economy", advantage.economy(startCost, runCost, lastYearFee));
        returnJsonObject.set("intelligence", advantage.intelligence());
        returnJsonObject.set("environment", advantage.environment(Double.valueOf(decimalFormat.format(calculateAnnualCarbon/1000))));


        return returnJsonObject;
    }

}
